#include <hive.h>

#define SPEED 4 /*Speed for jumping, falling, and running*/

/*Main loop functions*/
char init(void);
void deinit(void);
void handle_input(void);
void implement_physics(void);
void rndr(void);

/*Data for main caracter (you)*/
spr spr_you;
obj you;
int x,y;
int up;

/*Data for blocks*/
spr spr_block;
struct tblock
{
 int x,y;
 obj o;
 tblock *next;
};
typedef struct tblock block;
block *first;

/*These functions check if there is a "block" (wall/floor) is a respective direction, returning a pointer to the block in question*/
block *block_below(void);
block *block_above(void);
block *block_right(void);
block *block_left(void);

void rndr(void)
{
 block *b;
 cls();
 for(b=first;b!=NULL;b=b->next)
  rndr_obj(&(b->o),b->x,b->y);
 rndr_obj(&you,x,y);
}

void handle_input(void)
{
 block *blk;
 if(keydown(VK_UP)&&(NULL!=block_below()))
  up=24;
 if(keydown(VK_RIGHT)&&!block_right())
 {
  x+=SPEED;
 }
 if(keydown(VK_LEFT)&&!block_left())
  x-=SPEED;
}

void implement_physics(void)
{
 block *b;
 if(up<=0) /*Should we fall?*/
 {
  b=block_below();
  if(b==NULL) /*There's nothing below us! FALL!*/
   y+=SPEED;
  else
  {
   /*Move so we are exactly above the block*/
   y=b->y-32;
  }
 }
 else
 {
  if(NULL!=block_above()) /*Is something above us to hit our heads on?*/
   up=0;
  else /*Jump*/
  {
   y-=SPEED;
   up--;
  }
 }
}
   
block *block_below(void)
{
 block *b;
 /*Loop through the blocks*/
 for(b=first;b!=NULL;b=b->next)
  if(collision(&you,x,y,&(b->o),b->x,b->y)&&(y<b->y)&&(abs(x-b->x)<32))
   return(b);
 return(NULL);
}

block *block_above(void)
{
 block *b;
 /*Loop through the blocks*/
 for(b=first;b!=NULL;b=b->next)
  if(collision(&you,x,y,&(b->o),b->x,b->y)&&(y>b->y)&&(abs(x-b->x)<32))
   return(b);
 return(NULL);
}

block *block_right(void)
{
 block *b;
 /*Loop through the blocks*/
 for(b=first;b!=NULL;b=b->next)
  if(collision(&you,x,y,&(b->o),b->x,b->y)&&(x<b->x)&&(abs(y-b->y)<32))
   return(b);
 return(NULL);
}

block *block_left(void)
{
 block *b;
 /*Loop through the blocks*/
 for(b=first;b!=NULL;b=b->next)
  if(collision(&you,x,y,&(b->o),b->x,b->y)&&(x>b->x)&&(abs(y-b->y)<32))
   return(b);
 return(NULL);
}

char init(void)
{
 char wrld[]="X        XX        XX        XX        XX X      XX    X   XX        XX    X   XX        XXXXXXXXXXX";
 int xblock,yblock;
 block *b;
 char *c=&wrld[0];
 if(!start("Side scroller demo",30,1024,768,1))
  return(0);
 /*Load sprites*/
 if(!load_spr(&spr_you,1,1,"you.bmp","youmask.bmp"))
  return(0);
 if(!load_spr(&spr_block,1,1,"block.bmp",NULL))
  return(0);
 /*Assosiate objects*/
 init_obj(&you,&spr_you);
 /*Fill in data for you*/
 up=0;
 x=32*5;
 y=0;
 /*create blocks*/
 for(yblock=0;yblock<320;yblock+=32)
 {
  for(xblock=0;xblock<320;xblock+=32)
  {
   if(*c=='X')
   {
    b=(block *)malloc(sizeof(block));
    if(!first) /*Are we the first block?*/
     b->next=NULL;
    else /*Add us to the beggining*/
     b->next=first;
    first=b;
    init_obj(&(b->o),&spr_block);
    b->x=xblock;
    b->y=yblock;
   }
   c++; /*Advance character count (Pun not intended)*/
  }
 }
 return(1);
}

void deinit(void)
{
 kill_spr(&spr_you);
 kill_spr(&spr_block);
 end();
}

int main()
{
 if(!init())
 {
  popup("Error in init function.","");
  end();
  return(0);
 }
 
 /*Main loop*/
 while(1)
 {
  update();
  implement_physics();
  handle_input();
  rndr();
  if(keydown(VK_ESCAPE))
   break;
 }
 deinit();
 return(0);
}
